#!/bin/bash
echo "You are going to convert some VASP files into NAMD usable files!"
# You need to have: 
# - modified CM5PACK: First you need to get CM5PACK. Visit:
#   https://comp.chem.umn.edu/cm5pac/
#   Note that the newer versions (2015+) are capable of treating periodic systems.
#   This was not the case for the version used herein (2013), explaining the need for the warper below
#   Modify the source code as follows:
#   Line 3: PARAMETER (MAT=6000)
#   Comment line 18 to 46, except for 26-28.
#   Line 27: WRITE (*,'(I5,5X,F11.6,X,F11.6)') I,CCM5(I),CHIR(I)
#   Line 355: CCM5(J)=CCM5(J)*1.20D0
#   Recompile and put in your path or set absolute path accordingly in cm5_vasp.pl 
# - AMBERTools (tleap)
# - Linux environment, featuring awk, sed and perl
# - Few scripts called by this one:
#   * con2pdb.pl
#   * cm5_vasp.pl
# - A file called: 
#   surface.pdb  which contains the coordinates of the empty surface
#   To generate it, you can use point 1 & 2 of the subsequent script
# - A file called:
#   cm5_surf  which contains the corresponding charges. 
#   To generate it, you can use cm5_vasp.pl
# For "testing" purposes, we provide a file called CONTCAR_ads and an OUTCAR_ads, which are copied to CONTCAR and OUTCAR in the very beginning of this script:
cp CONTCAR_ads CONTCAR
cp OUTCAR_ads OUTCAR
# When playing with your own files, you have to change/comment these two lines
echo "Please be aware: This script is in no sense generally applicable"
echo "                 However, it outlines a general strategy that can be adapted to your specific case"
echo " Later versions might be published either in up-coming publications or on our website"

#1. Triple (in both in-plane directions) the VASP unit cell:
awk 'NR==1,NR==2 {print $0} 
     NR==3,NR==4 {printf "%.16f %.16f %.16f\n", 3*$1, 3*$2, 3*$3}
     NR==5 {printf  "%.16f %.16f %.16f\n", $1, $2, $3}
     NR==6 {print $0}
     NR==7 { for (i=1;i<=NF;i++) {printf "%.0f ", $i*9} printf "\n"}
     NR==8,NR==9 {print $0}
     NR>=10&&NF==6 {printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n", $1/3, $2/3, $3, $4, $5, $6, $1/3+1/3, $2/3, $3, $4, $5, $6, $1/3, $2/3+1/3, $3, $4, $5, $6, $1/3+1/3, $2/3+1/3, $3, $4, $5, $6;
    printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3, $3, $4, $5, $6, $1/3, $2/3 -1/3, $3, $4, $5, $6,$1/3 - 1/3,$2/3 -1/3, $3, $4, $5, $6;
 printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3 + 1/3, $3, $4, $5, $6, $1/3 + 1/3, $2/3 -1/3, $3, $4, $5, $6}
' CONTCAR > 9 

#2. Convert VASP format to pdb
./con2pdb.pl 9

#3. Use tleap to generate a solvated, cubic "cluster"
tleap -f cubic.in

#4. Remove water molecules that are not on top of the original hexagonal unit cell
awk '
function crossproduct(a, b, n){
n[1]=a[2]*b[3]-a[3]*b[2]
n[2]=a[3]*b[1]-a[1]*b[3]
n[3]=a[1]*b[2]-a[2]*b[1]
}
BEGIN{
a[1]=32.40;a[2]=0;a[3]=0
b[1]=16.20; b[2]=28.06  ; b[3]=0
}
{
if($1~/TER/ || $1~/END/){
if(i==3)
{
for(ii=0;ii<i;ii++){
print line[ii]
}
}
if(i==3 ||j==0){
print $0
}
i=0
}
else if($4!~/WAT/){
print $0
i++
}
else {
j=1
c[1]=$6-b[1]/2+a[1];c[2]=$7+b[2]/2;c[3]=0
 crossproduct(a,c,A)
crossproduct(b,c,B)
if((A[3]>0 && B[3]<0 )){
c[1]=$6-b[1]/2-b[1];c[2]=$7+b[2]/2-b[2];c[3]=0
 crossproduct(a,c,A)
c[1]=$6-b[1]/2;c[2]=$7+b[2]/2-a[2];c[3]=0
crossproduct(b,c,B)
if((A[3]<0 && B[3]>0 )){
line[i]=$0
i++
}}}}
' cubic.pdb > hex.pdb

#5. Use tleap to generate a parameter file
tleap -f hexagonal.in 

#6. Take care of a correct FEP setup (incoming and outgoing atoms (Alch), as well as frozen/flexible (f) molecules
awk '{
if(NF==10 && $1~/ATOM/) {
if($3 ~/Ru/ && i==2){i=1}
if(i!=1 &&$0!~/WAT/){
if($3 !~/Ru/){i=2}
f=1;Alch=1}
if(i==1 && $3 ~/Ru/){
f=1;Alch=-1}
if($0~/WAT/){
f=0;Alch=0}
printf  "%-6s%5d  %-3s %3s  %3d    %8.3f%8.3f%8.3f %5.2f %5.2f\n",$1,$2,$3,$4,$5,$6,$7,$8,f, Alch
}
else{print $0}
}' hex.pdb |  awk 'BEGIN{n=0;res=1}
{ if($0~/TER/){res++} if($0~/ATOM/){
printf  "%-6s%5d  %-3s %3s  %3d    %8.3f%8.3f%8.3f %5.2f %5.2f\n","ATOM",++n,$3,$4,res,$6,$7,$8,$9, $10}else{print $0}}'  > hexagonal.pdb 

#7. Extract CM5 Charges from VASP computations using CM5 pack and a wrapper that takes care of 3D replication
# CM5PACK cannot be distributed freely; please contact Don Truhlar at the University of Minnesota 
# (see also at the beginning of this script):
# For the 2013 version of cm5pack, one needs to replicate in 3 directions in order to "simulate" PBC:

awk 'NR==1,NR==2 {print $0} 
     NR==3,NR==4 {printf "%.16f %.16f %.16f\n", 3*$1, 3*$2, 3*$3}
     NR==5 {printf  "%.16f %.16f %.16f\n", 3*$1, 3*$2, 3*$3}
     NR==6 {print $0}
     NR==7 { for (i=1;i<=NF;i++) {printf "%.0f ", $i*27} printf "\n"}
     NR==8,NR==9 {print $0}
     NR>=10&&NF==6 {printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n", $1/3, $2/3, $3/3, $4, $5, $6, $1/3+1/3, $2/3, $3/3, $4, $5, $6, $1/3, $2/3+1/3, $3/3, $4, $5, $6, $1/3+1/3, $2/3+1/3, $3/3, $4, $5, $6;
    printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3, $3/3, $4, $5, $6, $1/3, $2/3 -1/3, $3/3, $4, $5, $6,$1/3 - 1/3,$2/3 -1/3, $3/3, $4, $5, $6;
 printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3 + 1/3, $3/3, $4, $5, $6, $1/3 + 1/3, $2/3 -1/3, $3/3, $4, $5, $6;

printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n", $1/3, $2/3, $3/3+1/3, $4, $5, $6, $1/3+1/3, $2/3, $3/3+1/3, $4, $5, $6, $1/3, $2/3+1/3, $3/3+1/3, $4, $5, $6, $1/3+1/3, $2/3+1/3, $3/3+1/3, $4, $5, $6;
printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3, $3/3+1/3, $4, $5, $6, $1/3, $2/3 -1/3, $3/3+1/3, $4, $5, $6,$1/3 - 1/3,$2/3 -1/3, $3/3+1/3, $4, $5, $6;
printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3 + 1/3, $3/3+1/3, $4, $5, $6, $1/3 + 1/3, $2/3 -1/3, $3/3+1/3, $4, $5, $6;

printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n", $1/3, $2/3, $3/3 - 1/3, $4, $5, $6, $1/3+1/3, $2/3, $3/3 - 1/3, $4, $5, $6, $1/3, $2/3+1/3, $3/3 - 1/3, $4, $5, $6, $1/3+1/3, $2/3+1/3, $3/3 - 1/3, $4, $5, $6;
printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n  %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3, $3/3 - 1/3, $4, $5, $6, $1/3, $2/3 -1/3, $3/3 - 1/3, $4, $5, $6,$1/3 - 1/3,$2/3 -1/3, $3/3 - 1/3, $4, $5, $6;
printf "%.16f %.16f %.16f %s %s %s\n %.16f %.16f %.16f %s %s %s\n",$1/3 - 1/3, $2/3 + 1/3, $3/3 - 1/3, $4, $5, $6, $1/3 + 1/3, $2/3 -1/3, $3/3 - 1/3, $4, $5, $6;
}
' CONTCAR > 3D


./cm5_vasp.pl

#8. Triple charges in each direction and convert to units as necessary for AMBER
awk '
BEGIN{
 if(ARGC==4){charge=ARGV[3];ARGC=3}
 while ((getline < "CONTCAR") > 0){{J++}
	if(J==7){
	for(i=1;i<=NF;i++){
        NAtoms+=$i
	N[i]=$i
}
}}
J=0
Nions=i
i=1
J=1
}

{
if($1>0 && FNR<=NAtoms)
{
if(ARGIND==1){Sys1[FNR]=$2
sum1+=$2
multiple=9
}
if(ARGIND==2){Sys2[$1]=$2
sum2+=$2
NAtoms2=$1
multiple2=9
}
}
}

END{
if((sum1 < 0.1 && sum1> -0.1) || charge!=0){
for(i=1;i<=NAtoms;i++){
Sys1[i]-=(sum1-charge)/NAtoms
}
}
if(sum2 < 0.1 && sum2> -0.1){
Sys2[1]-=sum2
}
J=1
for(i=1;i<=NAtoms;i++){
for(j=1;j<=multiple;j++){
sum+=Sys1[i]*18.2223
printf "%16.8e", Sys1[i]*18.2223
if(J%5==0){print ""}
J++
}
}
for(i=1;i<=NAtoms2;i++){
for(j=1;j<=multiple2;j++){
sum+=Sys2[i]*18.2223
printf "%16.8e", Sys2[i]*18.2223
if(J%5==0){print ""}
J++
}
}
}' cm5  cm5_surf > tmpC   

#9. Put these converted charges in the right place in the parameter file created by tleap
awk '{if($0~/-1.51973982/ && i!=0){
      if(J==0){print $0}
      else{
      for(j=1;j<=J;j++){
printf "%16.8e", a[j]
}
for(jj=j;jj<=5;jj++){
printf "%16.8e", $jj
}
printf "\n"
}
      i=0}
     else if($0~/CHARGE/){i=1;print $0}
     else if($0~/FORM/ && i==1){print $0 
J=0
 while ((getline < "tmpC") > 0){
	if(NF==5){
print $0
}
else{
J=NF
for(j=1;j<=J;j++){
a[j]=$j
}
}
}

}
     else if(i!=1){print $0}}' hexagonal.prmtop > hexagonalC.prmtop

echo "You are done! You can run NAMD computations using hexagonalC.prmtop as amber-style force field/topology/parameter file and hexagonal.pdb as the initial coordinate file!"
# Cleaning up the mess:
rm hex.pdb cm5 cm5.in solvated.prmtop solvated.rst7 leap.log 9 3D 3D.xyz 9.pdb tmpC

