#!/usr/bin/perl
use warnings;
use File::Basename;
# This script reads a file called "3D" and the associated "OUTCAR" of VASP
# and runs cm5pack to generate cm5 charges.

# Useful print formatting variables:
$,=" ";
$\="\n";

print "Using CONTCAR and OUTCAR information for CM5 charges";
print "Note: CM5 charges depend on geometry";
print "      We replicate the unit cell once in 3D and take the central box as the reference";
my $vbohr=0.529177 ;
# How many atoms
my $N;
# Convert VASP format to xyz
$inputfilename = "3D";
$outputfilename = "3D.xyz";
$inputfile = "";
open(IN,$inputfilename);
while(<IN>) { $inputfile .= $_; }
close(IN);
@inputfile = split(/\n/,$inputfile);
foreach(@inputfile) { $_ =~ s/^\s+//; }

$ntypes = $inputfile[5];
$natoms = $inputfile[6];
@natoms = split(/\s+/,$natoms);
@Ntypes= split(/\s+/,$ntypes);
$ntypes=$#Ntypes+1;
$totatoms = 0;
for($i=0; $i<$ntypes; $i++) {
    $totatoms += $natoms[$i];
}
if( $inputfile[8]=~/direct/i){
@comp=split(/\s+/,$inputfile[2]);
$scale=$inputfile[1];
@vx=@comp;
$norm=sqrt($comp[0]**2+$comp[1]**2+$comp[2]**2);
@comp=split(/\s+/,$inputfile[3]);
$norm=sqrt($comp[0]**2+$comp[1]**2+$comp[2]**2);
@vy=@comp;
@comp=split(/\s+/,$inputfile[4]);
@vz=@comp;
$norm=sqrt($comp[0]**2+$comp[1]**2+$comp[2]**2);
}
$outfile = $totatoms."\n";
$outfile .= "Generated with con2xyz\n";
$ln = 9;
for($type=0; $type<$ntypes; $type++) {
   if(0==1){ @line = split(/\s+/,$inputfile[$ln]);
    if(scalar(@line)>0) {
        $atype = $line[0];
    } else {
        $atype = "Type"."$type+1";
    }
    $ln += 2;}
    for($i=0; $i<$natoms[$type]; $i++) {
        $atype=$Ntypes[$type];
        @line = split(/\s+/,$inputfile[$ln]);
        $l[0]=$line[0]*$vx[0]+$line[1]*$vy[0]+$line[2]*$vz[0];
        $l[1]=$line[0]*$vx[1]+$line[1]*$vy[1]+$line[2]*$vz[1];
        $l[2]=$line[0]*$vx[2]+$line[1]*$vy[2]+$line[2]*$vz[2];
        $l[0]*=$scale;$l[1]*=$scale;$l[2]*=$scale;
        $outfile .= $atype."   ".join("   ",@l[0..2])."\n";
        $ln++;
    }
}
open (OUT,">$outputfilename");
print OUT $outfile;
close (OUT);

# Atom types
my $atoms;
my @atoms;
# Coordinates
my @x ; my @y ; my @z ;
my $i;my $j; my $l;
my %Z=(
H => 1, He => 2, 
Li => 3, Be => 4, B => 5, C => 6, N => 7, O => 8, F => 9, Ne => 10,
Na => 11, Mg => 12, Al => 13, Si => 14, P => 15, S => 16, Cl => 17, Ar => 18,
K => 19, Ca => 20,
Sc => 21, Ti => 22, V => 23, Cr => 24, Mn => 25, Fe => 26, Co => 27, Ni => 28, Cu => 29, Zn => 30,
Ga => 31, Ge => 32, As => 33, Se => 34, Br => 35, Kr => 36,
Rb => 37, Sr => 38,
Y => 39, Zr => 40, Nb => 41, Mo => 42, Tc => 43, Ru => 44, Rh => 45, Pd => 46, Ag => 47, Cd => 48,
In => 49, Sn => 50, Sb => 51, Te => 52, I => 53, Xe => 54,
Cs => 55, Ba => 56,
La => 57, Ce => 58, Pr => 59, Nd => 60, Pm => 61, Sm => 62, Eu => 63, Gd => 64, Tb => 65, Dy => 66, Ho => 67, Er => 68, Tm => 69, Yb => 70,
Lu => 71, Hf => 72, Ta => 73, W => 74, Re => 75, Os => 76, Ir => 77, Pt => 78, Au => 79, Hg => 80,
Tl => 81, Pb => 82, Bi => 83, Po => 84, At => 85, Rn => 86,
Fr => 87, Ra => 88,
Ac => 89, Th => 90, Pa => 91, U => 92, Np => 93, Pu => 94, Am => 95, Cm => 96, Bk => 97, Cf => 98, Es => 99, Fm => 100, Md => 101, No => 102,
);
my $argv="CONTCAR";
# First read the input
if(! -e $argv ){
die("File $argv does not exist!");
}
open(I, "<3D.xyz");
open(O, ">cm5.in");
my(@lines) = <I>;
$atoms=$lines[0];
printf O "%s", $atoms;
for($i=2;$i<=$atoms+1;$i++){
@_=split(/\s+/,$lines[$i]);
if($_[0]=~/[a-z]/i){
$j=0;
}
elsif($_[1]=~/[a-z]/i){
$j=1;
}
else{
$i=$i+1;
die("Check line $i");
}
$N="\L$_[$j]";
$N="\u$N";
printf O " %3d %13.6f %13.6f %13.6f\n", $Z{$N}, $_[$j+1], $_[$j+2], $_[$j+3]; 
}
close(I);
if(! -e "OUTCAR" ){
die("File OUTCAR does not exist!");
}

@lines=`sed -n '/Hirshfeld-DOM/,/^\$/p' OUTCAR | awk '{if(\$4~/[0-9]/)print \$4}'`;
if($#lines<2){
@lines=`sed -n '/Hirshfeld charges:/,/^\$/p' OUTCAR | awk '{if(\$3~/[0-9]/)print \$3}'`;
}
for($i=0;$i<$atoms/27;$i++){
for($j=0;$j<27; $j++){
printf O "%s",$lines[$i] ;
}
}
close(O);
@lines=`cm5pac.x < cm5.in`;
open(O, ">cm5");
for($i=0;$i<$atoms/27;$i++){
@_=split(/\s+/,$lines[$i*27]);
print O $i+1,$_[2],$_[3];
}
close(O)
